package DomainOperations::ResellerClubSOAP;

use warnings;
use strict;
=head1 NAME

DomainOperations::ResellerClubSOAP - A class to search and book a Domain Name via ResellerClub SOAP API!

=head1 VERSION

Version 0.02

=cut

our $VERSION = '0.02';

use base 'DomainOperations';
use Carp;

sub new{
	croak "Not implemented";	
}


=item C<new>
Not yet implemented, please dont use this function for now.

=head1 NOTE:

This module is provided as is, and is still underdevelopment, not suitable for Production use.

Virus free , Spam Free , Spyware Free Software and hopefully Money free software .


=head1 AUTHOR

<Abhishek jain>
goyali at cpan.org

=head1 SEE ALSO

http://www.ejain.com

=cut
1;    # End of DomainOperations::ResellerClubSOAP
